final: prev:

let
  pySmartDL = prev.callPackage ./development/python/pySmartDL {};
  cfscrape = prev.callPackage ./development/python/cfscrape {};
  system_hotkey = prev.callPackage ./development/python/system_hotkey {};
  discord-system-electron = prev.callPackage ./applications/networking/instant-messengers/discord {
    # Wayland
    # NOTE: electron under Wayland has no window titlebars
    extraStartupArgs = "--enable-features=UseOzonePlatform --ozone-platform=wayland";
    #extraStartupArgs = "--enable-features=UseOzonePlatform --ozone-platform=wayland --enable-features=WebRTCPipeWireCapturer";
  };
  wrapInSandbox = prev.callPackage ./sandbox.nix {};
in { 
  key-mapper = prev.callPackage ./tools/games/key-mapper {};
  vfio-isolate = prev.callPackage ./tools/virtualisation/vfio-isolate {};
  mopidy-funkwhale = prev.callPackage ./applications/audio/mopidy/funkwhale.nix {};
  # Not as clean as I hoped, but it works
  gnomeExtensions = prev.gnomeExtensions // {
    notification-timeout = prev.callPackage ./desktop/gnome/extensions/notification-timeout {};
    windowisready-remover = prev.callPackage ./desktop/gnome/extensions/windowisready-remover {};
  };
  python3Packages = prev.python3Packages // {
    pySmartDL = pySmartDL;
    cfscrape = cfscrape;
    system_hotkey = system_hotkey;
  };
  replaysorcery = prev.callPackage ./tools/games/replaysorcery {};
  evdev-proxy = prev.callPackage ./tools/virtualisation/evdev-proxy {};
  anime4k = prev.callPackage ./misc/video/anime4k {};
  anime-dl = prev.callPackage ./tools/video/animedl {
    pySmartDL = pySmartDL;
    cfscrape = cfscrape;
  };
  superpaper = prev.callPackage ./applications/desktop/superpaper {
    system_hotkey = system_hotkey;
  };
  discord-wrapped = wrapInSandbox {
    name = "discord";
    package = prev.discord;
    #package = discord-system-electron;
    binaryName = "discord";
    mountInHome = [ ".config/discord" ".config/BetterDiscord" ];
    additionalBlacklist = [ "/mnt" ];
    chdirTo = "/home/$USER";

    version = "1.0.1";
    copyIntoSandbox = [ "share/pixmaps" ];
    desktopFileArgs = {
      name = "Discord (wrapped)";
      #exec = "discord";
      icon = "discord";
      desktopName = "Discord (wrapped)";
      genericName = "Instant Messenger";
      categories = "Network;InstantMessaging;";
    };
  };

  sddm-sugar-dark = prev.callPackage ./sddm-sugar-dark {};
  
  iptsd = prev.callPackage ./tools/daemons/iptsd {};
  surface-control = prev.callPackage ./tools/system/surface-control {};

  trackma = prev.callPackage ./tools/video/trackma {};

  multimc-wrapped = wrapInSandbox {
    name = "multimc";
    package = prev.multimc;
    binaryName = "multimc";
    mountInHome = [ ".local/share/multimc" ];
    additionalBlacklist = [ "/mnt" ];
    chdirTo = "/home/$USER";
    extraEnv = {
      DRI_PRIME = "1";
    };
  };
  
  steam-wrapped = wrapInSandbox {
    name = "steam";
    package = prev.steam;
    binaryName = "steam";
    mountInHome = [ ".steam" ".local/share/Steam" ];
    additionalBlacklist = [ "/mnt" ];
    additionalMounts = [ "/mnt/Storage/Games/SteamLibrary" ];
    chdirTo = "/home/$USER";
    extraEnv = {
      DRI_PRIME = "1";
    };

    copyIntoSandbox = [ "/share" ];
  };

  android-studio-wrapped = wrapInSandbox {
    name = "android-studio";
    package = prev.android-studio;
    binaryName = "android-studio";
    additionalBlacklist = [ "/mnt" ];
    mountInHome = [ "Development/Personal/Android" ".android" ];
    chdirTo = "/home/$USER";
  };

  xournalpp-git = prev.xournalpp.overrideAttrs (old: {
    version = "1.1.1";
    src = prev.fetchFromGitHub {
      owner = "xournalpp";
      repo = "xournalpp";
      rev = "master";
      sha256 = "1scwnjzlp27vx5gqx8zk3zy9swwn0xhv550ig4nmjjx609ic1j7n";
    };
  });
  
  gajim-mainwindow = prev.callPackage ./applications/networking/instant-messengers/gajim {
    nbxmpp = prev.python3Packages.nbxmpp.overrideAttrs (old: {
      src = prev.fetchFromGitLab {
        domain = "dev.gajim.org";
        owner = "gajim";
        repo = "python-nbxmpp";
        rev = "master";
        sha256 = "1vifb68d47ihzm86iv4aqjh84wcxp5xmggfxwajwmls5mqi420yn";
      };
    });
  };

  swaync = prev.callPackage ./tools/desktop/swaync {};
}
